package fr.enssat.lancer;

import org.ini4j.Profile.Section;

public abstract class Objet3D
{
    /// couleur diffuse fixe
    protected Couleur Kd = new Couleur();

    /// couleur réfléchie fixe
    protected Couleur Ks = new Couleur();
    protected float ns = 0.0f;


    /**
     * calcule/retourne la couleur diffuse de this en ce point
     * @param point considéré
     * @return Couleur du point
     */
    public Couleur getKd(final Point point)
    {
        return Kd;
    }


    /**
     * affecte la couleur diffuse pour tout l'objet (lecture du fichier scene.ini)
     * @param Kd2 couleur à affecter
     */
    public void setKd(final Couleur Kd2)
    {
        Kd = Kd2;
    }


    /**
     * calcule/retourne la couleur spéculaire/reflets de this en ce point
     * @param point considéré
     * @return Couleur du point
     */
    public Couleur getKs(final Point point)
    {
        return Ks;
    }


    /**
     * affecte la couleur spéculaire/reflets pour tout l'objet (lecture du fichier scene.ini)
     * @param Kd2 couleur à affecter
     */
    public void setKs(final Couleur Ks2)
    {
        Ks = Ks2;
    }


    /**
     * calcule/retourne le coefficient spéculaire de this en ce point
     * @param point considéré
     * @return Couleur du point
     */
    public float getNs(final Point point)
    {
        return ns;
    }


    /**
     * affecte le coefficient spéculaire pour tout l'objet (lecture du fichier scene.ini)
     * @param Kd2 couleur à affecter
     */
    public void setNs(final float ns2)
    {
        ns = ns2;
    }


    /**
     * calcule la distance du point d'intersection entre this et le rayon
     * ne renvoie pas de point situé "derrière" le rayon
     * @param incident rayon considéré
     * @return Constantes.INFINI si pas d'intersection correcte
     */
    public abstract float getIntersectionDistance(final Rayon incident);


    /**
     * retourne le vecteur normal au point de contact donné par le rayon incident
     * @param incident
     * @return vecteur normalisé
     */
    public abstract Vecteur getNormale(final Point point);


    /**
     * calcule la couleur de l'objet au point désigné par contact
     * si l'objet est réfléchissant, on peut relancer au maximum
     * profondeur rayons indirects
     * @param scene
     * @param contact point considéré
     * @param profondeur nombre maximal d'appels récursifs de cette méthode
     * @return
     */
    public Couleur getColor(final Scene scene, final PointContact contact, final int profondeur)
    {
        // FIXME faire les calculs
        return new Couleur(1,1,0);
    }


    /**
     * extrait les informations de couleur de l'objet
     * @param sectionini
     */
    protected void parseIni(final Section sectionini)
    {
        // kd=<r> <v> <b>
        if (sectionini.containsKey("Kd")) {
            String[] mots = sectionini.get("Kd").trim().split("[ \t]");
            Kd = new Couleur(Float.parseFloat(mots[0]), Float.parseFloat(mots[1]), Float.parseFloat(mots[2]));
        }

        // Ks=<r> <v> <b> ou Kr=<r> <v> <b>
        if (sectionini.containsKey("Ks")) {
            String[] mots = sectionini.get("Ks").trim().split("[ \t]");
            Ks = new Couleur(Float.parseFloat(mots[0]), Float.parseFloat(mots[1]), Float.parseFloat(mots[2]));
        } else if (sectionini.containsKey("Kr")) {
            String[] mots = sectionini.get("Kr").trim().split("[ \t]");
            Ks = new Couleur(Float.parseFloat(mots[0]), Float.parseFloat(mots[1]), Float.parseFloat(mots[2]));
        }

        // ns=<n>
        if (sectionini.containsKey("ns")) {
            String[] mots = sectionini.get("ns").trim().split("[ \t]");
            ns = Float.parseFloat(mots[0]);
        }
    }

}
