package fr.enssat.lancer;

import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import org.ini4j.Ini;


/**
 * Cette classe représente l'ensemble des objets à dessiner ainsi que les lampes
 */
public class Scene
{
    // objets
    private final ArrayList<Objet3D> Objets = new ArrayList<>();

    // lumières
    private final ArrayList<Lampe> Lampes = new ArrayList<>();


    /**
     * cette méthode affecte objet, distance et contact avec le plus proche
     * objet rencontré par ce rayon. Elle passe en revue tous les objets sauf
     * celui qui est indiqué dans le paramètre sauflui.
     * Elle calcule les coordonnées du point de contact.
     * @param incident
     * @param sauflui
     * @return plus proche PointContact trouvé
     */
    public PointContact getClosestIntersection(final Rayon incident, final Objet3D sauflui)
    {
        // initialisation à aucun contact
        float bestDistance = Constantes.INFINI;
        Objet3D bestObjet = null;

        // FIXME programmer la recherche du plus proche objet intersecté par incident, sauf sauflui

        // retourne le contact ou null
        return incident.calcContact(bestDistance, bestObjet);
    }


    /**
     * cette méthode affecte objet, distance et contact avec le premier
     * objet rencontré par ce rayon. Elle passe en revue tous les objets sauf
     * celui qui est indiqué dans le paramètre sauflui.
     * Elle calcule les coordonnées du point de contact.
     * @param incident
     * @param sauflui
     * @return premier PointContact trouvé
     */
    public PointContact getAnyIntersection(final Rayon incident, final Objet3D sauflui)
    {
        // FIXME programmer la recherche d'un objet intersecté par incident, n'importe lequel, sauf sauflui
        // retourne le PointContact s'il y a un contact, null sinon

        // aucun contact
        return null;
    }


    /**
     * constructeur : charge un fichier de description d'une scène
     * @param nom du fichier scène à charger
     */
    public Scene(final String nom) throws IOException
    {
        readINI(nom);
    }

    private void readINI(final String nom) throws IOException
    {
        // ouvrir le fichier ini avec la bibliothèque
        Ini ini = new Ini();
        ini.load(new FileReader(nom));

        // parcourir toutes les section [Type n°]
        for (String type: ini.keySet()) {

            if ("config".equalsIgnoreCase(type)) {
                // lecture de la configuration
                Constantes.read(ini.get(type));
            } else

            switch (type.split(" ", 2)[0].toLowerCase()) {

                case "sphere":
                    Objets.add(new Sphere(ini.get(type)));
                    break;

                case "lampe":
                    Lampes.add(new Lampe(ini.get(type)));
                    break;
            }
        }
    }


    /**
     * retourne la liste des lampes
     * @return liste des lampes
     */
    public final ArrayList<Lampe> getLampes()
    {
        return Lampes;
    }
}
